@@ -1,9 +1,10 @@
 // Test the vtkDICOMReader by displaying an image.
 
 #include "vtkDICOMMetaData.h"
-#include "vtkDICOMSorter.h"
+#include "vtkDICOMDirectory.h"
 #include "vtkDICOMReader.h"
 #include "vtkDICOMCTRectifier.h"
+#include "vtkDICOMApplyPalette.h"
 
 #include "vtkRenderWindowInteractor.h"
 #include "vtkInteractorStyleImage.h"
@@ -12,7 +13,7 @@
 #include "vtkCamera.h"
 #include "vtkImageData.h"
 #include "vtkImageReslice.h"
-#include "vtkImageSliceMapper.h"
+#include "vtkImageResliceMapper.h"
 #include "vtkImageProperty.h"
 #include "vtkImageSlice.h"
 #include "vtkImageReader2.h"
@@ -21,6 +22,7 @@
 #include "vtkIntArray.h"
 #include "vtkMatrix4x4.h"
 #include "vtkMath.h"
+#include "vtkErrorCode.h"
 
 int main(int argc, char *argv[])
 {
@@ -34,9 +36,6 @@ int main(int argc, char *argv[])
   iren->SetRenderWindow(renWin);
   iren->SetInteractorStyle(style);
 
-  vtkSmartPointer<vtkDICOMSorter> sorter =
-    vtkSmartPointer<vtkDICOMSorter>::New();
-
   vtkSmartPointer<vtkStringArray> files =
     vtkSmartPointer<vtkStringArray>::New();
 
@@ -57,37 +56,35 @@ int main(int argc, char *argv[])
       }
     }
 
+  // find all DICOM files supplied by the user
+  vtkSmartPointer<vtkDICOMDirectory> sorter =
+    vtkSmartPointer<vtkDICOMDirectory>::New();
+  sorter->RequirePixelDataOn();
+  sorter->SetScanDepth(1);
   sorter->SetInputFileNames(files);
   sorter->Update();
 
-#if 0
   // find the series with the largest number of files
   int m = sorter->GetNumberOfStudies();
-  int studyIdx = 0;
   int seriesIdx = 0;
   int kmax = 0;
   for (int i = 0; i < m; i++)
     {
-    int fj = sorter->GetFirstSeriesInStudy(i);
-    int lj = fj + sorter->GetNumberOfSeriesInStudy(i);
-    for (int j = fj; j < lj; j++)
+    int fj = sorter->GetFirstSeriesForStudy(i);
+    int lj = sorter->GetLastSeriesForStudy(i);
+    for (int j = fj; j <= lj; j++)
       {
       int k = sorter->GetFileNamesForSeries(j)->GetNumberOfValues();
       if (k > kmax)
         {
         kmax = k;
         seriesIdx = j;
-        studyIdx = i;
         }
       }
     }
 
   // display the longest series
-  //vtkStringArray *a = sorter->GetFileNamesForSeries(seriesIdx);
-#endif
-
-  // trust the user and display all the files, even if multiple series
-  vtkStringArray *a = sorter->GetOutputFileNames();
+  vtkStringArray *a = sorter->GetFileNamesForSeries(seriesIdx);
   vtkSmartPointer<vtkImageReslice> reslice =
     vtkSmartPointer<vtkImageReslice>::New();
   vtkSmartPointer<vtkDICOMReader> reader =
@@ -104,73 +101,31 @@ int main(int argc, char *argv[])
   double range[2];
   int extent[6];
   reader->Update();
+
+  if (reader->GetErrorCode() != vtkErrorCode::NoError)
+    {
+    return 1;
+    }
+
   reader->GetOutput()->GetScalarRange(range);
   reader->GetOutput()->GetExtent(extent);
 
   // get the output port to connect to the display pipeline
   vtkAlgorithmOutput *portToDisplay = reader->GetOutputPort();
 
-#if 0
-  // check if the matrix indicates a tilted CT gantry
-  vtkSmartPointer<vtkMatrix4x4> pmat =
-    vtkSmartPointer<vtkMatrix4x4>::New();
-  pmat->DeepCopy(reader->GetPatientMatrix());
-  double xvec[4] = { 1.0, 0.0, 0.0, 0.0 };
-  double yvec[4] = { 0.0, 1.0, 0.0, 0.0 };
-  double zvec[4] = { 0.0, 0.0, 1.0, 0.0 };
-  pmat->MultiplyPoint(xvec, xvec);
-  pmat->MultiplyPoint(yvec, yvec);
-  pmat->MultiplyPoint(zvec, zvec);
+  vtkSmartPointer<vtkDICOMApplyPalette> palette =
+    vtkSmartPointer<vtkDICOMApplyPalette>::New();
+  palette->SetInputConnection(reader->GetOutputPort());
+  palette->Update();
+  palette->GetOutput()->GetScalarRange(range);
+  portToDisplay = palette->GetOutputPort();
 
-  // create slice normal and compare to zvec
-  double normal[3];
-  vtkMath::Cross(xvec, yvec, normal);
-  double checkvec[3];
-  vtkMath::Cross(zvec, normal, checkvec);
-  if (vtkMath::Norm(checkvec) > 1e-3)
-    {
-    // definite gantry tilt, the volume is trapezoid-shaped
-    vtkSmartPointer<vtkMatrix4x4> rmat =
-      vtkSmartPointer<vtkMatrix4x4>::New();
-    rmat->DeepCopy(pmat);
-    rmat->SetElement(0, 2, normal[0]);
-    rmat->SetElement(1, 2, normal[1]);
-    rmat->SetElement(2, 2, normal[2]);
-    pmat->Invert();
-    vtkMatrix4x4::Multiply4x4(pmat, rmat, rmat);
-
-    // pure shear matrix will have only one element that is different
-    // from the identity matrix:
-    double shear = rmat->GetElement(1, 2);
-    rmat->Identity();
-    rmat->SetElement(1, 2, shear);
-
-    double origin[3], spacing[3];
-    reader->GetOutput()->GetOrigin(origin);
-    reader->GetOutput()->GetSpacing(spacing);
-    // adjust the origin to centre the new volume on the old trapezoid
-    origin[1] -= shear*0.5*spacing[2]*(extent[5] - extent[4]);
-
-    // use vtkImageReslice to eliminate any shear
-    reslice->SetOutputOrigin(origin);
-    reslice->SetOutputSpacing(spacing);
-    reslice->SetOutputExtent(extent);
-    reslice->SetResliceAxes(rmat);
-    reslice->SetInputConnection(reader->GetOutputPort());
-    reslice->SetInterpolationModeToLinear();
-    reslice->Update();
-
-    // specify the port that the display pipeline will use
-    portToDisplay = reslice->GetOutputPort();
-    }
-#else
   vtkSmartPointer<vtkDICOMCTRectifier> rect =
     vtkSmartPointer<vtkDICOMCTRectifier>::New();
   rect->SetVolumeMatrix(reader->GetPatientMatrix());
-  rect->SetInputConnection(reader->GetOutputPort());
+  rect->SetInputConnection(portToDisplay);
   rect->Update();
   portToDisplay = rect->GetOutputPort();
-#endif
 
   static double viewport[3][4] = {
     { 0.67, 0.0, 1.0, 0.5 },
@@ -183,14 +138,15 @@ int main(int argc, char *argv[])
 
   for (int i = 2*(imageIs3D == 0); i < 3; i++)
     {
-    vtkSmartPointer<vtkImageSliceMapper> imageMapper =
-      vtkSmartPointer<vtkImageSliceMapper>::New();
+    vtkSmartPointer<vtkImageResliceMapper> imageMapper =
+      vtkSmartPointer<vtkImageResliceMapper>::New();
     if (i < 3)
       {
       imageMapper->SetInputConnection(portToDisplay);
       }
-    imageMapper->SetOrientation(i % 3);
+    imageMapper->SliceFacesCameraOn();
     imageMapper->SliceAtFocalPointOn();
+    imageMapper->ResampleToScreenPixelsOn();
 
     vtkSmartPointer<vtkImageSlice> image =
       vtkSmartPointer<vtkImageSlice>::New();
@@ -226,9 +182,9 @@ int main(int argc, char *argv[])
 
     vtkCamera *camera = renderer->GetActiveCamera();
     camera->SetFocalPoint(point);
-    point[imageMapper->GetOrientation()] -= 500.0;
+    point[i % 3] -= 500.0;
     camera->SetPosition(point);
-    if (imageMapper->GetOrientation() == 2)
+    if ((i % 3) == 2)
       {
       camera->SetViewUp(0.0, -1.0, 0.0);
       }
