@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2014 David Gobbi
+  Copyright (c) 2012-2015 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -12,14 +12,19 @@
 
 =========================================================================*/
 
+#include "vtkDICOMConfig.h"
 #include "vtkDICOMParser.h"
-#include "vtkDICOMSorter.h"
+#include "vtkDICOMDictionary.h"
+#include "vtkDICOMDirectory.h"
 #include "vtkDICOMMetaData.h"
 #include "vtkDICOMItem.h"
+#include "vtkDICOMUtilities.h"
 
 // from dicomcli
+#include "mainmacro.h"
 #include "readquery.h"
 
+#include <vtkSortFileNames.h>
 #include <vtkStringArray.h>
 #include <vtkSmartPointer.h>
 
@@ -40,7 +45,7 @@ void printVersion(FILE *file, const char *cp)
 {
   fprintf(file, "%s %s\n", cp, DICOM_VERSION);
   fprintf(file, "\n"
-    "Copyright (c) 2012-2014, David Gobbi.\n\n"
+    "Copyright (c) 2012-2015, David Gobbi.\n\n"
     "This software is distributed under an open-source license.  See the\n"
     "Copyright.txt file that comes with the vtk-dicom source distribution.\n");
 }
@@ -51,9 +56,10 @@ void printUsage(FILE *file, const char *cp)
   fprintf(file, "usage:\n"
     "  %s file1.dcm [file2.dcm ...]\n\n", cp);
   fprintf(file, "options:\n"
+    "  -k tag          Provide a key to be printed.\n"
     "  -q <query.txt>  Provide a file that lists which elements to print.\n"
-    "  --help      Print a brief help message.\n"
-    "  --version   Print the software version.\n");
+    "  --help          Print a brief help message.\n"
+    "  --version       Print the software version.\n");
 }
 
 // print the help
@@ -61,7 +67,15 @@ void printHelp(FILE *file, const char *cp)
 {
   printUsage(file, cp);
   fprintf(file, "\n"
-    "Dump the metadata from a DICOM series as text.\n");
+    "Dump the metadata from a DICOM series as text.  If only a few\n"
+    "attributes are to be dumped, then their tags can be given with \"-k\"\n"
+    "(the \"-k\" option can be repeated as many times as needed).  Tags can\n"
+    "given in hexadecimal GGGG,EEEE format, or in text format as specified\n"
+    "in the DICOM dictionary.  Alternately, the tags can be listed in a\n"
+    "query file given with the \"-q\" option (one tag per line).\n"
+    "Attributes nested within sequences can be specified by giving a tag\n"
+    "path e.g. \"-k Tag1/Tag2/Tag3\".  Either a forward slash or a backslash\n"
+    "can be used to separate the components of the path.\n\n");
 }
 
 // remove path portion of filename
@@ -75,7 +89,8 @@ const char *fileBasename(const char *filename)
 // Print out one data element
 void printElement(
   vtkDICOMMetaData *meta, const vtkDICOMItem *item,
-  const vtkDICOMDataElementIterator &iter, int depth)
+  const vtkDICOMDataElementIterator &iter, int depth,
+  unsigned int pixelDataVL)
 {
   vtkDICOMTag tag = iter->GetTag();
   int g = tag.GetGroup();
@@ -137,36 +152,50 @@ void printElement(
     {
     v = vp[vi];
     unsigned int vl = v.GetVL();
+    if (tag == DC::PixelData ||
+        tag == DC::FloatPixelData ||
+        tag == DC::DoubleFloatPixelData)
+      {
+      vl = (depth == 0 && vl == 0 ? pixelDataVL : v.GetVL());
+      }
     std::string s;
-    if (v.GetVR() == vtkDICOMVR::UN ||
-        v.GetVR() == vtkDICOMVR::SQ)
+    if (vr == vtkDICOMVR::UN ||
+        vr == vtkDICOMVR::SQ)
       {
       // sequences are printed later
       s = (vl > 0 ? "..." : "");
       }
-    else if (v.GetVR() == vtkDICOMVR::LT ||
-             v.GetVR() == vtkDICOMVR::ST ||
-             v.GetVR() == vtkDICOMVR::UT)
+    else if (vr == vtkDICOMVR::LT ||
+             vr == vtkDICOMVR::ST ||
+             vr == vtkDICOMVR::UT)
       {
       // replace breaks with "\\", cap length to MAX_LENGTH
+      size_t l = (vl > MAX_LENGTH ? MAX_LENGTH-4 : vl);
       const char *cp = v.GetCharData();
-      unsigned int j = 0;
-      while (j < vl && cp[j] != '\0')
+      std::string utf8;
+      if (v.GetCharacterSet() != vtkDICOMCharacterSet::ISO_IR_6)
         {
-        unsigned int k = j;
-        unsigned int m = j;
-        for (; j < vl && cp[j] != '\0'; j++)
+        utf8 = v.GetCharacterSet().ConvertToUTF8(cp, l);
+        l = utf8.length();
+        cp = utf8.data();
+        }
+      size_t j = 0;
+      while (j < l && cp[j] != '\0')
+        {
+        size_t k = j;
+        size_t m = j;
+        for (; j < l && cp[j] != '\0'; j++)
           {
           m = j;
           if (cp[j] == '\r' || cp[j] == '\n' || cp[j] == '\f')
             {
             do { j++; }
-            while (j < vl && (cp[j] == '\r' || cp[j] == '\n' || cp[j] == '\f'));
+            while (j < l && (cp[j] == '\r' || cp[j] == '\n' || cp[j] == '\f'));
             break;
             }
           m++;
           }
-        if (j == vl)
+        if (j == l)
           {
           while (m > 0 && cp[m-1] == ' ') { m--; }
           }
@@ -175,9 +204,8 @@ void printElement(
           s.append("\\\\");
           }
         s.append(&cp[k], m-k);
-        if (s.size() > MAX_LENGTH)
+        if (vl > MAX_LENGTH)
           {
-          s.resize(MAX_LENGTH-4);
           s.append("...");
           break;
           }
@@ -205,54 +233,188 @@ void printElement(
         }
       }
 
-    if (vi == 0)
+    if (meta && vi == 0)
       {
       printf("%s(%04X,%04X) %s \"%s\" :", indent, g, e, vr.GetText(), name);
       }
-    if (vn > 1)
+    if (meta && vn > 1)
       {
       printf("%s%s  %4.4u",
         (vi == 0 ? " (multiple values)\n" : ""), indent, vi + 1);
       }
-    if (v.GetVR() == vtkDICOMVR::SQ)
+    if (vr == vtkDICOMVR::SQ)
       {
-      unsigned int m = v.GetNumberOfValues();
+      size_t m = v.GetNumberOfValues();
       const vtkDICOMItem *items = v.GetSequenceData();
       printf(" (%u %s%s)\n",
-        m, (m == 1 ? "item" : "items"),
-       (v.GetVL() == 0xffffffffu ? ", delimited" : ""));
-      for (unsigned int j = 0; j < m; j++)
+        static_cast<unsigned int>(m), (m == 1 ? "item" : "items"),
+        (vl == 0xffffffffu ? ", delimited" : ""));
+      for (size_t j = 0; j < m; j++)
         {
         printf("%s%s---- SQ Item %04u at offset %u ----\n",
-          indent, spaces+(MAX_INDENT - INDENT_SIZE), j+1,
+          indent, spaces+(MAX_INDENT - INDENT_SIZE),
+          static_cast<unsigned int>(j+1),
           items[j].GetByteOffset());
         vtkDICOMDataElementIterator siter = items[j].Begin();
         vtkDICOMDataElementIterator siterEnd = items[j].End();
 
         for (; siter != siterEnd; ++siter)
           {
-          printElement(meta, &items[j], siter, depth+1);
+          printElement(meta, &items[j], siter, depth+1, pixelDataVL);
           }
         }
       }
-    else if (v.GetVL() == 0xffffffffu)
+    else if (vl == 0xffffffffu)
       {
-      printf(" [%s] (delimited)\n", s.c_str());
+      if (tag == DC::PixelData ||
+          tag == DC::FloatPixelData ||
+          tag == DC::DoubleFloatPixelData)
+        {
+        printf(" [...] (compressed)\n");
+        }
+      else
+        {
+        printf(" [...] (delimited)\n");
+        }
       }
     else
       {
-      printf(" [%s] (%u bytes)\n", s.c_str(), v.GetVL());
+      const char *uidName = "";
+      if (vr == vtkDICOMVR::UI)
+        {
+        uidName = vtkDICOMUtilities::GetUIDName(s.c_str());
+        }
+      if (uidName[0] != '\0')
+        {
+        printf(" [%s] {%s} (%u bytes)\n", s.c_str(), uidName, vl);
+        }
+      else if (vr == vtkDICOMVR::OB ||
+               vr == vtkDICOMVR::OW ||
+               vr == vtkDICOMVR::OF ||
+               vr == vtkDICOMVR::OD)
+        {
+        printf(" [%s] (%u bytes)\n", (vl == 0 ? "" : "..."), vl);
+        }
+      else
+        {
+        printf(" [%s] (%u bytes)\n", s.c_str(), vl);
+        }
+      }
+    }
+}
+
+void printElementFromTagPathRecurse(
+  const vtkDICOMItem *item, const vtkDICOMTagPath& tagPath,
+  const vtkDICOMTagPath& fullPath, int *count)
+{
+  vtkDICOMTag tag = tagPath.GetHead();
+
+  vtkDICOMDataElementIterator iter = item->Begin();
+  while (iter != item->End() && iter->GetTag() != tag)
+    {
+    ++iter;
+    }
+
+  if (iter != item->End())
+    {
+    if (tagPath.HasTail())
+      {
+      const vtkDICOMItem *items = iter->GetValue().GetSequenceData();
+      if (items)
+        {
+        size_t n = iter->GetValue().GetNumberOfValues();
+        for (size_t i = 0; i < n; i++)
+          {
+          printElementFromTagPathRecurse(
+            &items[i], tagPath.GetTail(), fullPath, count);
+          }
+        }
+      }
+    else
+      {
+      ++(*count);
+      fprintf(stdout, "  %04d", *count);
+      printElement(0, item, iter, 0, 0);
+      }
+    }
+}
+
+void printElementFromTagPath(
+  vtkDICOMMetaData *data, const vtkDICOMTagPath& tagPath,
+  unsigned int pixelDataVL)
+{
+  int count = 0;
+  vtkDICOMTag tag = tagPath.GetHead();
+  vtkDICOMDataElementIterator iter = data->Find(tag);
+  if (iter != data->End())
+    {
+    if (tagPath.HasTail())
+      {
+      for (vtkDICOMTagPath p = tagPath; ; p = p.GetTail())
+        {
+        int g = p.GetHead().GetGroup();
+        int e = p.GetHead().GetElement();
+        if (p.HasTail())
+          {
+          printf("(%04X,%04X)\\", g, e);
+          }
+        else
+          {
+          vtkDICOMDictEntry entry =
+            vtkDICOMDictionary::FindDictEntry(p.GetHead());
+          vtkDICOMVR vr = entry.GetVR();
+          const char *name = entry.GetName();
+          name = (name ? name : "Unknown");
+          printf("(%04X,%04X) %s \"%s\" : (nested)\n",
+            g, e, vr.GetText(), name);
+          break;
+          }
+        }
+      if (iter->IsPerInstance())
+        {
+        for (int j = 0; j < iter->GetNumberOfInstances(); j++)
+          {
+          const vtkDICOMItem *items = iter->GetValue(j).GetSequenceData();
+          if (items)
+            {
+            size_t n = iter->GetValue().GetNumberOfValues();
+            for (size_t i = 0; i < n; i++)
+              {
+              printElementFromTagPathRecurse(
+                &items[i], tagPath.GetTail(), tagPath, &count);
+              }
+            }
+          }
+        }
+      else
+        {
+        const vtkDICOMItem *items = iter->GetValue().GetSequenceData();
+        if (items)
+          {
+          size_t n = iter->GetValue().GetNumberOfValues();
+          for (size_t i = 0; i < n; i++)
+            {
+            printElementFromTagPathRecurse(
+              &items[i], tagPath.GetTail(), tagPath, &count);
+            }
+          }
+        }
+      }
+    else
+      {
+      printElement(data, 0, iter, 0, pixelDataVL);
       }
     }
 }
 
 // This program will dump all the metadata in the given file
-int main(int argc, char *argv[])
+int MAINMACRO(int argc, char *argv[])
 {
   int rval = 0;
 
   // for the optional query file
-  const char *qfile = 0;
+  QueryTagList qtlist;
+  vtkDICOMItem query;
 
   if (argc < 2)
     {
@@ -287,7 +449,12 @@ int main(int argc, char *argv[])
 
       if (arg[1] == 'q')
         {
-        qfile = argv[++argi];
+        const char *qfile = argv[++argi];
+        if (!dicomcli_readquery(qfile, &query, &qtlist))
+          {
+          fprintf(stderr, "Can't read query file %s\n\n", qfile);
+          return 1;
+          }
         }
       else if (arg[1] == 'o')
         {
@@ -295,6 +462,21 @@ int main(int argc, char *argv[])
         return 1;
         }
       }
+    else if (strcmp(arg, "-k") == 0)
+      {
+      vtkDICOMTag tag;
+      ++argi;
+      if (argi == argc)
+        {
+        fprintf(stderr, "%s must be followed by gggg,eeee=value "
+                        "where gggg,eeee is a DICOM tag.\n\n", arg);
+        return 1;
+        }
+      if (!dicomcli_readkey(argv[argi], &query, &qtlist))
+        {
+        return 1;
+        }
+      }
     else if (arg[0] == '-')
       {
       fprintf(stderr, "unrecognized option %s.\n\n", arg);
@@ -307,34 +489,17 @@ int main(int argc, char *argv[])
       }
     }
 
-  // read the query file, create a query
-  vtkDICOMMetaData *query = 0;
-  if (qfile)
-    {
-    vtkDICOMItem qitem = dicomcli_readquery(qfile);
-    vtkDICOMDataElementIterator iter = qitem.Begin();
-    vtkDICOMDataElementIterator iterEnd = qitem.End();
-    query = vtkDICOMMetaData::New();
-    while (iter != iterEnd)
-      {
-      query->SetAttributeValue(iter->GetTag(), iter->GetValue());
-      ++iter;
-      }
-    }
-
   // sort the files by study and series
-  vtkSmartPointer<vtkDICOMSorter> sorter =
-    vtkSmartPointer<vtkDICOMSorter>::New();
+  vtkSmartPointer<vtkDICOMDirectory> sorter =
+    vtkSmartPointer<vtkDICOMDirectory>::New();
   sorter->RequirePixelDataOff();
+  sorter->SetScanDepth(0);
+  sorter->SetFindQuery(query);
   sorter->SetInputFileNames(files);
   sorter->Update();
 
   vtkSmartPointer<vtkDICOMParser> parser =
     vtkSmartPointer<vtkDICOMParser>::New();
-  if (query)
-    {
-    parser->SetQuery(query);
-    }
 
   vtkSmartPointer<vtkDICOMMetaData> data =
     vtkSmartPointer<vtkDICOMMetaData>::New();
@@ -349,33 +514,62 @@ int main(int argc, char *argv[])
   int m = sorter->GetNumberOfStudies();
   for (int j = 0; j < m; j++)
     {
-    int k = sorter->GetFirstSeriesInStudy(j);
-    int n = sorter->GetNumberOfSeriesInStudy(j);
-    n += k;
-    for (; k < n; k++)
+    int k = sorter->GetFirstSeriesForStudy(j);
+    int kl = sorter->GetLastSeriesForStudy(j);
+    for (; k <= kl; k++)
       {
       vtkStringArray *a = sorter->GetFileNamesForSeries(k);
       vtkIdType l = a->GetNumberOfValues();
       std::string fname = a->GetValue(0);
-      printf("=========== %s =========\n", fileBasename(fname.c_str()));
+      if (l == 1)
+        {
+        printf("==== %s ====\n", fname.c_str());
+        }
+      else
+        {
+        // print the first and last filenames (sorted)
+        vtkSmartPointer<vtkSortFileNames> fsort =
+          vtkSmartPointer<vtkSortFileNames>::New();
+        fsort->NumericSortOn();
+        fsort->IgnoreCaseOn();
+        fsort->GroupingOff();
+        fsort->SetInputFileNames(a);
+        fsort->Update();
+        printf("==== %s .. %s (%d files) ====\n",
+          fsort->GetFileNames()->GetValue(0).c_str(),
+          fileBasename(fsort->GetFileNames()->GetValue(l-1).c_str()),
+          static_cast<int>(l));
+        }
 
       data->Clear();
       data->SetNumberOfInstances(static_cast<int>(l));
 
+      unsigned int pixelDataVL = 0;
       for (vtkIdType i = 0; i < l; i++)
         {
         fname = a->GetValue(i);
         parser->SetIndex(i);
         parser->SetFileName(fname.c_str());
         parser->Update();
+        pixelDataVL = parser->GetPixelDataVL();
         }
 
-      vtkDICOMDataElementIterator iter = data->Begin();
-      vtkDICOMDataElementIterator iterEnd = data->End();
-
-      for (; iter != iterEnd; ++iter)
+      if (query.GetNumberOfDataElements() > 0)
         {
-        printElement(data, 0, iter, 0);
+        for (size_t i = 0; i < qtlist.size(); i++)
+          {
+          printElementFromTagPath(data, qtlist[i], pixelDataVL);
+          }
+        }
+      else
+        {
+        vtkDICOMDataElementIterator iter = data->Begin();
+        vtkDICOMDataElementIterator iterEnd = data->End();
+
+        for (; iter != iterEnd; ++iter)
+          {
+          printElement(data, 0, iter, 0, pixelDataVL);
+          }
         }
       }
     }
@@ -385,10 +579,5 @@ int main(int argc, char *argv[])
   SetConsoleOutputCP(codePage);
 #endif
 
-  if (query)
-    {
-    query->Delete();
-    }
-
   return rval;
 }
