@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2014 David Gobbi
+  Copyright (c) 2012-2015 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -12,13 +12,18 @@
 
 =========================================================================*/
 
+#include "vtkDICOMConfig.h"
 #include "vtkDICOMBuild.h"
+#include "vtkDICOMDictionary.h"
 #include "vtkDICOMMetaData.h"
 #include "vtkDICOMParser.h"
 #include "vtkDICOMReader.h"
-#include "vtkDICOMSorter.h"
+#include "vtkDICOMFileSorter.h"
 #include "vtkDICOMToRAS.h"
 #include "vtkDICOMCTRectifier.h"
+#include "vtkDICOMFile.h"
+#include "vtkDICOMFileDirectory.h"
+#include "vtkDICOMFilePath.h"
 #include "vtkNIFTIHeader.h"
 #include "vtkNIFTIWriter.h"
 
@@ -27,6 +32,7 @@
 #include <vtkMatrix4x4.h>
 #include <vtkImageReslice.h>
 #include <vtkImageCast.h>
+#include <vtkImageExtractComponents.h>
 #include <vtkStringArray.h>
 #include <vtkIntArray.h>
 #include <vtkErrorCode.h>
@@ -37,10 +43,6 @@
 #include <vtkImageHistogramStatistics.h>
 #endif
 
-#include <vtksys/SystemTools.hxx>
-#include <vtksys/Directory.hxx>
-#include <vtksys/Glob.hxx>
-
 #include <string>
 #include <vector>
 #include <set>
@@ -50,6 +52,9 @@
 #include <stdlib.h>
 #include <ctype.h>
 
+// from dicomcli
+#include "mainmacro.h"
+
 // Simple structure for command-line options
 struct dicomtonifti_options
 {
@@ -66,6 +71,7 @@ struct dicomtonifti_options
   bool batch;
   bool silent;
   bool verbose;
+  int volume;
   const char *output;
 };
 
@@ -80,7 +86,7 @@ void dicomtonifti_version(FILE *file, const char *command_name, bool verbose)
     {
     fprintf(file, "%s %s\n", cp, DICOM_VERSION);
     fprintf(file, "\n"
-      "Copyright (c) 2012-2014, David Gobbi.\n\n"
+      "Copyright (c) 2012-2015, David Gobbi.\n\n"
       "This software is distributed under an open-source license.  See the\n"
       "Copyright.txt file that comes with the vtk-dicom source distribution.\n");
     }
@@ -111,7 +117,7 @@ void dicomtonifti_usage(FILE *file, const char *command_name)
     "  -s --silent             Do not echo output filenames.\n"
     "  -v --verbose            Verbose error reporting.\n"
     "  -L --follow-symlinks    Follow symbolic links when recursing.\n"
-    "  --fsl                   Reformat the image for use in FSL.\n"
+    "  --fsl                   Format axial image for use in FSL.\n"
     "  --reformat-to-axial     Reformat the image into axial orientation.\n"
     "  --no-slice-reordering   Never reorder the slices.\n"
     "  --no-row-reordering     Never reorder the rows.\n"
@@ -119,6 +125,7 @@ void dicomtonifti_usage(FILE *file, const char *command_name)
     "  --no-reordering         Never reorder slices, rows, or columns.\n"
     "  --no-qform              Don't include a qform in the NIFTI file.\n"
     "  --no-sform              Don't include an sform in the NIFTI file.\n"
+    "  --volume N              Set the volume to output (starts at 0).\n"
     "  --version               Print the version and exit.\n"
     "  --build-version         Print source and build version.\n"
     "  --help                  Documentation for dicomtonifti.\n"
@@ -166,15 +173,22 @@ void dicomtonifti_help(FILE *file, const char *command_name)
     "type from unsigned 16-bit to signed 16-bit if necessary.\n"
     "\n");
   fprintf(file,
-    "If batch mode is enabled, then the filenames will automatically be\n"
-    "generated from the series description in the DICOM meta data:\n"
-    "\"PatientName/StudyDescription-ID/SeriesDescription_N.nii.gz\".\n"
+    "If batch mode is selected, the output file given with \"-o\" can be\n"
+    "constructed from DICOM attributes, by providing the attribute names\n"
+    "within curly braces.  For example, consider the following:\n"
+    "\"{PatientID}-{StudyDate}/{SeriesDescription}-{SeriesNumber}.nii\"\n"
+    "or something similar to produce a hierarchichal directory structure.\n"
+    "The attributes used in the path should be from the following list:\n"
+    "  PatientID, PatientName, PatientBirthDate, PatientSex,\n"
+    "  StudyID, StudyDescription, StudyDate, StudyTime, StudyInstanceUID,\n"
+    "  SeriesNumber, SeriesDescription, SeriesInstanceUID,\n"
+    "  Modality, AccessionNumber.\n"
     "\n"
     "Here is an example of batch mode that recurses into subdirectories\n"
     "and compresses the output files, putting the results in the current\n"
     "directory:\n"
     "\n"
-    "%s -brz -o . /path/to/dicom/files\n"
+    "%s -brz -o {SeriesDescription}-{SeriesNumber}.nii /path/to/dicom/files\n"
     "\n",
     command_name);
 }
@@ -183,7 +197,7 @@ void dicomtonifti_help(FILE *file, const char *command_name)
 void dicomtonifti_check_error(vtkObject *o)
 {
   vtkDICOMReader *reader = vtkDICOMReader::SafeDownCast(o);
-  vtkDICOMSorter *sorter = vtkDICOMSorter::SafeDownCast(o);
+  vtkDICOMFileSorter *sorter = vtkDICOMFileSorter::SafeDownCast(o);
   vtkNIFTIWriter *writer = vtkNIFTIWriter::SafeDownCast(o);
   vtkDICOMParser *parser = vtkDICOMParser::SafeDownCast(o);
   const char *filename = 0;
@@ -246,60 +260,6 @@ void dicomtonifti_check_error(vtkObject *o)
   exit(1);
 }
 
-// Add a dicom file to the list, expand if wildcard
-void dicomtonifti_add_file(vtkStringArray *files, const char *filepath)
-{
-#ifdef _WIN32
-  bool ispattern = false;
-  bool hasbackslash = false;
-  size_t n = strlen(filepath);
-  for (size_t i = 0; i < n; i++)
-    {
-    if (filepath[i] == '*' || filepath[i] == '?' || filepath[i] == '[')
-      {
-      ispattern = true;
-      }
-    if (filepath[i] == '\\')
-      {
-      hasbackslash = true;
-      }
-    }
-
-  std::string newpath = filepath;
-  if (hasbackslash)
-    {
-    // backslashes interfere with vtksys::Glob
-    vtksys::SystemTools::ConvertToUnixSlashes(newpath);
-    }
-  filepath = newpath.c_str();
-
-  if (ispattern)
-    {
-    vtksys::Glob glob;
-    if (glob.FindFiles(filepath))
-      {
-      const std::vector<std::string> &globfiles = glob.GetFiles();
-      size_t m = globfiles.size();
-      for (size_t j = 0; j < m; j++)
-        {
-        files->InsertNextValue(globfiles[j]);
-        }
-      }
-    else
-      {
-      fprintf(stderr, "Could not match pattern: %s\n", filepath);
-      exit(1);
-      }
-    }
-  else
-    {
-    files->InsertNextValue(filepath);
-    }
-#else
-  files->InsertNextValue(filepath);
-#endif
-}
-
 // Read the options
 void dicomtonifti_read_options(
   int argc, char *argv[],
@@ -318,6 +278,7 @@ void dicomtonifti_read_options(
   options->batch = false;
   options->silent = false;
   options->verbose = false;
+  options->volume = -1;
   options->output = 0;
 
   // read the options from the command line
@@ -390,6 +351,17 @@ void dicomtonifti_read_options(
         {
         options->verbose = true;
         }
+      else if (strcmp(arg, "--volume") == 0)
+        {
+        if (argi >= argc || argv[argi][0] == '-')
+          {
+          fprintf(stderr, "\nA number must follow \'--volume\'\n\n");
+          dicomtonifti_usage(stderr, argv[0]);
+          exit(1);
+          }
+        arg = argv[argi++];
+        options->volume = atoi(arg);
+        }
       else if (strcmp(arg, "--version") == 0)
         {
         dicomtonifti_version(stdout, argv[0], false);
@@ -469,78 +441,122 @@ void dicomtonifti_read_options(
       }
     else
       {
-      dicomtonifti_add_file(files, arg);
+      files->InsertNextValue(arg);
       }
     }
 
   while (argi < argc)
     {
-    dicomtonifti_add_file(files, argv[argi++]);
+    files->InsertNextValue(argv[argi++]);
     }
 }
 
-// Remove all characters but A-ZA-z0-9_ from a string
-std::string dicomtonifti_safe_string(const std::string& str)
+// Remove all characters but A-Za-z0-9_ from a string
+std::string dicomtonifti_safe_string(const std::string& input)
 {
-  std::string out;
-
-  size_t n = str.size();
-  size_t m = 0;
-  for (size_t i = 0; i < n; i++)
+  std::string::const_iterator a = input.begin();
+  std::string::const_iterator b = a;
+  std::string s;
+  while (a != input.end())
     {
-    char c = str[i];
-    if (!isalnum(c))
+    while (a != input.end() &&
+           ((*a & 0x80) == 0) && isgraph(*a) && !ispunct(*a))
       {
-      c = '_';
+      ++a;
       }
-    else
+    s.append(b, a);
+    b = a;
+    while (a != input.end() &&
+           (((*a & 0x80) != 0) || !isgraph(*a) || ispunct(*a)))
       {
-      m = i + 1;
+      ++a;
       }
-    out.push_back(c);
+    if (b != a && b != input.begin() && a != input.end())
+      {
+      s.append("_");
+      }
+    b = a;
     }
 
-  out.resize(m);
-
-  if (out.size() == 0)
-    {
-    out = "UNKNOWN";
-    }
-
-  return out;
+  return s;
 }
 
 // Generate an output filename from meta data
 std::string dicomtonifti_make_filename(
-  const char *outpath, vtkDICOMMetaData *meta)
+  const char *outfile, vtkDICOMMetaData *meta)
 {
-  std::string patientName = dicomtonifti_safe_string(
-    meta->GetAttributeValue(DC::PatientName).AsString());
-  std::string patientID = dicomtonifti_safe_string(
-    meta->GetAttributeValue(DC::PatientID).AsString());
-  std::string studyDesc = dicomtonifti_safe_string(
-    meta->GetAttributeValue(DC::StudyDescription).AsString());
-  std::string studyID = dicomtonifti_safe_string(
-    meta->GetAttributeValue(DC::StudyID).AsString());
-  std::string seriesDesc = dicomtonifti_safe_string(
-    meta->GetAttributeValue(DC::SeriesDescription).AsString());
-  std::string seriesNumber = dicomtonifti_safe_string(
-    meta->GetAttributeValue(DC::SeriesNumber).AsString());
+  std::string s;
+  std::string key;
+  std::string val;
+  vtkDICOMValue v;
 
-  if (patientName != "UNKNOWN")
+  const char *cp = outfile;
+  const char *dp = cp;
+  const char *bp = 0;
+  while (*cp != '\0')
     {
-    patientID = patientName;
+    while (*cp != '{' && *cp != '}' && *cp != '\0') { cp++; }
+    if (*cp == '}')
+      {
+      fprintf(stderr, "Missing \'{\': %s\n", outfile);
+      exit(1);
+      }
+    if (*cp == '{')
+      {
+      bp = cp;
+      while (*cp != '}' && *cp != '\0') { cp++; }
+      if (*cp != '}')
+        {
+        fprintf(stderr, "Unmatched \'{\': %s\n", outfile);
+        exit(1);
+        }
+      else
+        {
+        s.append(dp, bp);
+        bp++;
+        key.assign(bp, cp);
+        cp++;
+        dp = cp;
+        v.Clear();
+        vtkDICOMTag tag;
+        if (key.length() > 0)
+          {
+          vtkDICOMDictEntry de = vtkDICOMDictionary::FindDictEntry(key.c_str());
+          if (de.IsValid())
+            {
+            tag = de.GetTag();
+            }
+          else
+            {
+            fprintf(stderr, "Unrecognized key %s\n", key.c_str());
+            exit(1);
+            }
+          }
+        if (meta)
+          {
+          v = meta->GetAttributeValue(tag);
+          }
+        if (v.IsValid())
+          {
+          val.assign(dicomtonifti_safe_string(v.AsUTF8String()));
+          }
+        else if (meta)
+          {
+          fprintf(stderr, "Sorry, key %s not found.\n",
+                  key.c_str());
+          exit(1);
+          }
+        if (val.empty())
+          {
+          val = "Empty";
+          }
+        s.append(val);
+        }
+      }
     }
+  s.append(dp, cp);
 
-  std::vector<std::string> sv;
-
-  vtksys::SystemTools::SplitPath(outpath, sv);
-
-  sv.push_back(patientID);
-  sv.push_back(studyDesc + "-" + studyID);
-  sv.push_back(seriesDesc + "_" + seriesNumber + ".nii");
-
-  return vtksys::SystemTools::JoinPath(sv);
+  return s;
 }
 
 // Convert one DICOM series into one NIFTI file
@@ -561,6 +577,24 @@ void dicomtonifti_convert_one(
   vtkAlgorithmOutput *lastOutput = reader->GetOutputPort();
   vtkMatrix4x4 *patientMatrix = reader->GetPatientMatrix();
 
+  // extract just one volume, if requested
+  vtkSmartPointer<vtkImageExtractComponents> extract =
+    vtkSmartPointer<vtkImageExtractComponents>::New();
+  extract->SetInputConnection(lastOutput);
+  if (options->volume >= 0)
+    {
+    if (reader->GetOutput()->GetNumberOfScalarComponents() <= options->volume)
+      {
+      fprintf(stderr, "Only %d volumes, but --volume %d used.\n",
+              reader->GetOutput()->GetNumberOfScalarComponents(),
+              options->volume);
+      exit(1);
+      }
+    extract->SetComponents(options->volume);
+    extract->Update();
+    lastOutput = extract->GetOutputPort();
+    }
+
   // check if slices were reordered by the reader
   vtkIntArray *fileIndices = reader->GetFileIndexArray();
   vtkIntArray *frameIndices = reader->GetFrameIndexArray();
@@ -854,7 +888,8 @@ void dicomtonifti_convert_one(
   writer->SetDescription(descrip.c_str());
   writer->SetNIFTIHeader(hdr);
   writer->SetFileName(outfile);
-  if (reader->GetTimeDimension() > 1)
+  if (reader->GetTimeDimension() > 1 &&
+      options->volume < 0)
     {
     writer->SetTimeDimension(reader->GetTimeDimension());
     writer->SetTimeSpacing(reader->GetTimeSpacing());
@@ -892,8 +927,8 @@ void dicomtonifti_convert_files(
   presorter->Update();
 
   // sort the files by study and series
-  vtkSmartPointer<vtkDICOMSorter> sorter =
-    vtkSmartPointer<vtkDICOMSorter>::New();
+  vtkSmartPointer<vtkDICOMFileSorter> sorter =
+    vtkSmartPointer<vtkDICOMFileSorter>::New();
   sorter->SetInputFileNames(presorter->GetFileNames());
   sorter->Update();
   dicomtonifti_check_error(sorter);
@@ -904,10 +939,9 @@ void dicomtonifti_convert_files(
     if (options->compress)
       {
       size_t os = strlen(outpath);
-      if (os > 2 &&
-          (outpath[os-3] != '.' ||
-           tolower(outpath[os-2]) != 'g' ||
-           tolower(outpath[os-1]) != 'z'))
+      if (os < 3 || outpath[os-3] != '.' ||
+          tolower(outpath[os-2]) != 'g' ||
+          tolower(outpath[os-1]) != 'z')
         {
         outfile.append(".gz");
         }
@@ -926,10 +960,9 @@ void dicomtonifti_convert_files(
     int m = sorter->GetNumberOfStudies();
     for (int j = 0; j < m; j++)
       {
-      int k = sorter->GetFirstSeriesInStudy(j);
-      int n = sorter->GetNumberOfSeriesInStudy(j);
-      n += k;
-      for (; k < n; k++)
+      int k = sorter->GetFirstSeriesForStudy(j);
+      int kl = sorter->GetLastSeriesForStudy(j);
+      for (; k <= kl; k++)
         {
         // get metadata of first file
         vtkStringArray *a = sorter->GetFileNamesForSeries(k);
@@ -943,17 +976,23 @@ void dicomtonifti_convert_files(
         std::string outfile =
           dicomtonifti_make_filename(outpath, meta);
 
-        if (options->compress)
+        size_t os = outfile.length();
+        if (options->compress &&
+            (os < 3 || outfile[os-3] != '.' ||
+             tolower(outfile[os-2]) != 'g' ||
+             tolower(outfile[os-1]) != 'z'))
           {
           outfile.append(".gz");
           }
 
         // make the directory for the file
-        if (k == sorter->GetFirstSeriesInStudy(j))
+        if (k == sorter->GetFirstSeriesForStudy(j))
           {
-          std::string dirname = vtksys::SystemTools::GetParentDirectory(
-            outfile.c_str());
-          if (!vtksys::SystemTools::MakeDirectory(dirname.c_str()))
+          vtkDICOMFilePath path(outfile);
+          path.PopBack();
+          std::string dirname = path.AsString();
+          int code = vtkDICOMFileDirectory::Create(dirname.c_str());
+          if (code != vtkDICOMFileDirectory::Good)
             {
             fprintf(stderr, "Cannot create directory: %s\n",
                     dirname.c_str());
@@ -987,14 +1026,14 @@ void dicomtonifti_files_and_dirs(
   for (vtkIdType i = 0; i < n; i++)
     {
     std::string fname = files->GetValue(i);
+    vtkDICOMFilePath path(fname);
     size_t m = fname.size();
     if ((m > 1 && (fname[m-1] == '/' || fname[m-1] == '\\')) ||
-        vtksys::SystemTools::FileIsDirectory(fname.c_str()))
+        path.IsDirectory())
       {
       if (pastdirs->size() == 0 ||
           (options->recurse &&
-           (options->follow_symlinks ||
-            !vtksys::SystemTools::FileIsSymlink(fname.c_str()))))
+           (options->follow_symlinks || !path.IsSymlink())))
         {
         directories->InsertNextValue(fname.c_str());
         }
@@ -1011,35 +1050,34 @@ void dicomtonifti_files_and_dirs(
     }
 
   n = directories->GetNumberOfValues();
-  vtksys::Directory directory;
-  std::vector<std::string> pathparts;
   for (vtkIdType i = 0; i < n; i++)
     {
     std::string dirname = directories->GetValue(i);
+    vtkDICOMFilePath path(dirname);
 
     // avoid infinite recursion
-    std::string realpath = vtksys::SystemTools::GetRealPath(dirname.c_str());
+    std::string realpath = path.GetRealPath();
     if (pastdirs->count(realpath)) { continue; }
     pastdirs->insert(pastdirs->end(), realpath);
 
-    if (!directory.Load(dirname.c_str()))
+    vtkDICOMFileDirectory directory(dirname.c_str());
+    int code = directory.GetError();
+    if (code != vtkDICOMFileDirectory::Good)
       {
       fprintf(stderr, "Could not open directory %s\n", dirname.c_str());
       }
     else
       {
       files->Initialize();
-      vtksys::SystemTools::SplitPath(directory.GetPath(), pathparts);
       unsigned long nf = directory.GetNumberOfFiles();
       for (unsigned long j = 0; j < nf; j++)
         {
         const char *dirfile = directory.GetFile(j);
         if (dirfile[0] != '.')
           {
-          pathparts.push_back(dirfile);
-          std::string fullpath = vtksys::SystemTools::JoinPath(pathparts);
-          files->InsertNextValue(fullpath);
-          pathparts.pop_back();
+          path.PushBack(dirfile);
+          files->InsertNextValue(path.AsString());
+          path.PopBack();
           }
         }
       dicomtonifti_files_and_dirs(options, files, outpath, pastdirs);
@@ -1048,7 +1086,7 @@ void dicomtonifti_files_and_dirs(
 }
 
 // This program will convert DICOM to NIFTI
-int main(int argc, char *argv[])
+int MAINMACRO(int argc, char *argv[])
 {
   // for the list of input DICOM files
   vtkSmartPointer<vtkStringArray> files =
@@ -1070,14 +1108,16 @@ int main(int argc, char *argv[])
     exit(1);
     }
 
-  bool isDirectory = vtksys::SystemTools::FileIsDirectory(outpath);
+  int code = vtkDICOMFile::Access(outpath, vtkDICOMFile::In);
   size_t l = strlen(outpath);
-  if (options.batch && !isDirectory)
+  vtkDICOMFilePath tmp(outpath);
+  if (options.batch && code == vtkDICOMFile::FileIsDirectory)
     {
-    fprintf(stderr, "In batch mode, -o must give an existing directory.\n");
-    exit(1);
+    tmp.PushBack(
+      "{PatientID}-{StudyDate}-{SeriesDescription}-{SeriesNumber}.nii");
+    outpath = tmp.AsString().c_str();
     }
-  else if (!options.batch && (isDirectory ||
+  else if (!options.batch && (code == vtkDICOMFile::FileIsDirectory ||
            (l > 0 && (outpath[l-1] == '/' || outpath[l-1] == '\\'))))
     {
     fprintf(stderr, "The -o option must give a file, not a directory.\n");
