@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2014 David Gobbi
+  Copyright (c) 2012-2015 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -24,6 +24,9 @@
 #include <string.h>
 #include <stdlib.h>
 
+// from dicomcli
+#include "mainmacro.h"
+
 // remove path portion of filename
 const char *basename(const char *filename)
 {
@@ -33,7 +36,7 @@ const char *basename(const char *filename)
 }
 
 // This program will convert
-int main(int argc, char *argv[])
+int MAINMACRO(int argc, char *argv[])
 {
   int rval = 0;
 
