@@ -2,7 +2,7 @@
 
   Program: DICOM for VTK
 
-  Copyright (c) 2012-2014 David Gobbi
+  Copyright (c) 2012-2015 David Gobbi
   All rights reserved.
   See Copyright.txt or http://dgobbi.github.io/bsd3.txt for details.
 
@@ -12,17 +12,20 @@
 
 =========================================================================*/
 
+#include "vtkDICOMConfig.h"
 #include "vtkDICOMBuild.h"
 #include "vtkDICOMMetaData.h"
 #include "vtkDICOMParser.h"
 #include "vtkDICOMReader.h"
 #include "vtkDICOMWriter.h"
-#include "vtkDICOMSorter.h"
+#include "vtkDICOMFileSorter.h"
 #include "vtkDICOMMRGenerator.h"
 #include "vtkDICOMCTGenerator.h"
 #include "vtkDICOMToRAS.h"
 #include "vtkDICOMCTRectifier.h"
 #include "vtkDICOMUtilities.h"
+#include "vtkDICOMFile.h"
+#include "vtkDICOMFileDirectory.h"
 #include "vtkNIFTIHeader.h"
 #include "vtkNIFTIReader.h"
 
@@ -41,10 +44,6 @@
 #include <vtkImageHistogramStatistics.h>
 #endif
 
-#include <vtksys/SystemTools.hxx>
-#include <vtksys/Directory.hxx>
-#include <vtksys/Glob.hxx>
-
 #include <string>
 #include <vector>
 #include <set>
@@ -54,6 +53,9 @@
 #include <stdlib.h>
 #include <ctype.h>
 
+// from dicomcli
+#include "mainmacro.h"
+
 // Kinds of reformats
 enum MPREnum
 {
@@ -88,7 +90,7 @@ void niftitodicom_version(FILE *file, const char *command_name, bool verbose)
     {
     fprintf(file, "%s %s\n", cp, DICOM_VERSION);
     fprintf(file, "\n"
-      "Copyright (c) 2012-2014, David Gobbi.\n\n"
+      "Copyright (c) 2012-2015, David Gobbi.\n\n"
       "This software is distributed under an open-source license.  See the\n"
       "Copyright.txt file that comes with the vtk-dicom source distribution.\n");
     }
@@ -166,7 +168,7 @@ void niftitodicom_help(FILE *file, const char *command_name)
 void niftitodicom_check_error(vtkObject *o)
 {
   vtkNIFTIReader *reader = vtkNIFTIReader::SafeDownCast(o);
-  vtkDICOMSorter *sorter = vtkDICOMSorter::SafeDownCast(o);
+  vtkDICOMFileSorter *sorter = vtkDICOMFileSorter::SafeDownCast(o);
   vtkDICOMWriter *writer = vtkDICOMWriter::SafeDownCast(o);
   vtkDICOMParser *parser = vtkDICOMParser::SafeDownCast(o);
   const char *filename = 0;
@@ -229,60 +231,6 @@ void niftitodicom_check_error(vtkObject *o)
   exit(1);
 }
 
-// Add a dicom file to the list, expand if wildcard
-void niftitodicom_add_file(vtkStringArray *files, const char *filepath)
-{
-#ifdef _WIN32
-  bool ispattern = false;
-  bool hasbackslash = false;
-  size_t n = strlen(filepath);
-  for (size_t i = 0; i < n; i++)
-    {
-    if (filepath[i] == '*' || filepath[i] == '?' || filepath[i] == '[')
-      {
-      ispattern = true;
-      }
-    if (filepath[i] == '\\')
-      {
-      hasbackslash = true;
-      }
-    }
-
-  std::string newpath = filepath;
-  if (hasbackslash)
-    {
-    // backslashes interfere with vtksys::Glob
-    vtksys::SystemTools::ConvertToUnixSlashes(newpath);
-    }
-  filepath = newpath.c_str();
-
-  if (ispattern)
-    {
-    vtksys::Glob glob;
-    if (glob.FindFiles(filepath))
-      {
-      const std::vector<std::string> &globfiles = glob.GetFiles();
-      size_t m = globfiles.size();
-      for (size_t j = 0; j < m; j++)
-        {
-        files->InsertNextValue(globfiles[j]);
-        }
-      }
-    else
-      {
-      fprintf(stderr, "Could not match pattern: %s\n", filepath);
-      exit(1);
-      }
-    }
-  else
-    {
-    files->InsertNextValue(filepath);
-    }
-#else
-  files->InsertNextValue(filepath);
-#endif
-}
-
 // Check that a file has a NIFTI name
 bool isNIFTIFileName(const char *f)
 {
@@ -447,7 +395,7 @@ void niftitodicom_read_options(
       }
     else
       {
-      niftitodicom_add_file(files, arg);
+      files->InsertNextValue(arg);
       vtkIdType m = files->GetMaxId();
       if (m >= 0)
         {
@@ -472,7 +420,7 @@ void niftitodicom_read_options(
 
   while (argi < argc)
     {
-    niftitodicom_add_file(files, argv[argi++]);
+    files->InsertNextValue(argv[argi++]);
     vtkIdType m = files->GetMaxId();
     if (m >= 0)
       {
@@ -865,8 +813,8 @@ void niftitodicom_convert_files(
   presorter->Update();
 
   // sort the files by study and series
-  vtkSmartPointer<vtkDICOMSorter> sorter =
-    vtkSmartPointer<vtkDICOMSorter>::New();
+  vtkSmartPointer<vtkDICOMFileSorter> sorter =
+    vtkSmartPointer<vtkDICOMFileSorter>::New();
   sorter->SetInputFileNames(presorter->GetFileNames());
   sorter->Update();
   niftitodicom_check_error(sorter);
@@ -876,7 +824,7 @@ void niftitodicom_convert_files(
 }
 
 // This program will convert NIFTI to DICOM
-int main(int argc, char *argv[])
+int MAINMACRO(int argc, char *argv[])
 {
   // for the list of input DICOM files
   vtkSmartPointer<vtkStringArray> files =
@@ -911,15 +859,14 @@ int main(int argc, char *argv[])
     exit(1);
     }
 
-  if (vtksys::SystemTools::FileExists(outpath))
+  int code = vtkDICOMFile::Access(outpath, vtkDICOMFile::In);
+  if (code != vtkDICOMFile::FileIsDirectory)
     {
-    if (!vtksys::SystemTools::FileIsDirectory(outpath))
-      {
-      fprintf(stderr, "option -o must give a directory, not a file.\n");
-      exit(1);
-      }
+    fprintf(stderr, "option -o must give a directory, not a file.\n");
+    exit(1);
     }
-  else if (!vtksys::SystemTools::MakeDirectory(outpath))
+  code = vtkDICOMFileDirectory::Create(outpath);
+  if (code != vtkDICOMFileDirectory::Good)
     {
     fprintf(stderr, "Cannot create directory: %s\n", outpath);
     exit(1);
