@@ -69,6 +69,18 @@ int main(int argc, char *argv[])
   TestAssert(v.GetNumberOfValues() == 4);
   TestAssert(v.GetVL() == 16);
   TestAssert(memcmp(v.GetFloatData(), flt, 16) == 0);
+  // string values converted to tags
+  static const char *tagstring = "(ff00,0123)\\(0FA2,0001)";
+  v = vtkDICOMValue(vtkDICOMVR::AT, tagstring, strlen(tagstring));
+  TestAssert(v.GetNumberOfValues() == 2);
+  TestAssert(v.GetVL() == 8);
+  TestAssert(v.GetTag(0) == vtkDICOMTag(0xff00,0x0123));
+  TestAssert(v.GetTag(1) == vtkDICOMTag(0x0fa2,0x0001));
+  // tags constructed from enums
+  v = vtkDICOMValue(vtkDICOMVR::AT, DC::FrameTimeVector);
+  TestAssert(v.GetNumberOfValues() == 1);
+  TestAssert(v.GetVL() == 4);
+  TestAssert(v.AsTag() == vtkDICOMTag(0x0018,0x1065));
   // these text VRs should always report 1 value
   const char *hp = "he\\llo";
   size_t sl = strlen(hp);
@@ -286,10 +298,27 @@ int main(int argc, char *argv[])
   TestAssert(v.AsInt() == 3);
   v = vtkDICOMValue(vtkDICOMVR::UT, "3\\2\\1");
   TestAssert(v.GetNumberOfValues() == 1);
-  TestAssert(v.AsString() == "3\\2\\1 ");
+  TestAssert(v.AsString() == "3\\2\\1");
   TestAssert(v.AsInt() == 0);
   }
 
+  { // test AsUTF8String
+  vtkDICOMValue v;
+  std::string s;
+  // backslash interpreted as backslash
+  v = vtkDICOMValue(
+    vtkDICOMVR::LO, vtkDICOMCharacterSet("ISO 2022 IR 13\\ISO 2022 IR 87"),
+    "\xd4\xcf\xc0\xde^\xc0\xdb\xb3\\\x1b$B;3ED\x1b(J^\x1b$BB@O:\x1b(J");
+  s = "\xef\xbe\x94\xef\xbe\x8f\xef\xbe\x80\xef\xbe\x9e^\xef\xbe\x80\xef\xbe\x9b\xef\xbd\xb3\\\xe5\xb1\xb1\xe7\x94\xb0^\xe5\xa4\xaa\xe9\x83\x8e";
+  TestAssert(v.AsUTF8String() == s);
+  // backslash interpreted according to character set
+  v = vtkDICOMValue(
+    vtkDICOMVR::LT, vtkDICOMCharacterSet("ISO 2022 IR 13\\ISO 2022 IR 87"),
+    "\xd4\xcf\xc0\xde^\xc0\xdb\xb3\\\x1b$B;3ED\x1b(J^\x1b$BB@O:\x1b(J");
+  s = "\xef\xbe\x94\xef\xbe\x8f\xef\xbe\x80\xef\xbe\x9e^\xef\xbe\x80\xef\xbe\x9b\xef\xbd\xb3\xc2\xa5\xe5\xb1\xb1\xe7\x94\xb0^\xe5\xa4\xaa\xe9\x83\x8e";
+  TestAssert(v.AsUTF8String() == s);
+  }
+
   { // test Matches for query matching
   vtkDICOMValue v;
   vtkDICOMValue u;
@@ -297,7 +326,7 @@ int main(int argc, char *argv[])
   // test comparison of null values
   TestAssert(v.Matches(u));
   u = vtkDICOMValue(vtkDICOMVR::CS, "HELLO");
-  TestAssert(v.Matches(u));
+  TestAssert(!v.Matches(u));
   TestAssert(u.Matches(v));
 
   // test comparison of empty values
@@ -334,6 +363,14 @@ int main(int argc, char *argv[])
   TestAssert(!v.Matches(u));
   u = vtkDICOMValue(vtkDICOMVR::CS, "H*P");
   TestAssert(!v.Matches(u));
+  u = vtkDICOMValue(vtkDICOMVR::CS, "HELL*");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(vtkDICOMVR::CS, "HELLO*");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(vtkDICOMVR::CS, "HELL?");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(vtkDICOMVR::CS, "HELLO?");
+  TestAssert(!v.Matches(u));
 
   // test multiple values in the query
   v = vtkDICOMValue(vtkDICOMVR::UI, "10.3000.11.6");
@@ -484,9 +521,15 @@ int main(int argc, char *argv[])
   sv.AddItem(item2);
   v = sv;
   TestAssert(v.Matches(u));
+  vtkDICOMValue ve;
   vtkDICOMSequence su2;
   u = su2;
   TestAssert(v.Matches(u));
+  TestAssert(ve.Matches(u));
+  su2.AddItem(vtkDICOMItem());
+  u = su2;
+  TestAssert(v.Matches(u));
+  TestAssert(ve.Matches(u));
 
   // test comparisons of encoded strings
   v = vtkDICOMValue(vtkDICOMVR::SH, vtkDICOMCharacterSet::ISO_IR_100,
@@ -525,6 +568,48 @@ int main(int argc, char *argv[])
   TestAssert(!v.Matches(u));
   u = vtkDICOMValue(vtkDICOMVR::PN, names[5]);
   TestAssert(!v.Matches(u));
+
+  // test comparison with character sets
+  std::string s;
+  v = vtkDICOMValue(
+    vtkDICOMVR::LO, vtkDICOMCharacterSet("ISO 2022 IR 13\\ISO 2022 IR 87"),
+    "\xd4\xcf\xc0\xde^\xc0\xdb\xb3\\\x1b$B;3ED\x1b(J^\x1b$BB@O:\x1b(J");
+  u = vtkDICOMValue(
+    vtkDICOMVR::LO, vtkDICOMCharacterSet("ISO_IR 13"),
+    "\xd4\xcf\xc0\xde^\xc0\xdb\xb3");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(
+    vtkDICOMVR::LO, vtkDICOMCharacterSet("ISO 2022 IR 13\\ISO 2022 IR 87"),
+    "\x1b$B;3ED\x1b(J^\x1b$BB@O:\x1b(J");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(
+    vtkDICOMVR::LO, vtkDICOMCharacterSet("ISO_IR 192"),
+    "\xef\xbe\x94\xef\xbe\x8f\xef\xbe\x80\xef\xbe\x9e^\xef\xbe\x80\xef\xbe\x9b\xef\xbd\xb3");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(
+    vtkDICOMVR::LO, vtkDICOMCharacterSet("ISO_IR 192"),
+    "\xe5\xb1\xb1\xe7\x94\xb0^\xe5\xa4\xaa\xe9\x83\x8e");
+
+  // test comparison with character sets specifically for PN
+  v = vtkDICOMValue(
+    vtkDICOMVR::PN, vtkDICOMCharacterSet("ISO 2022 IR 13\\ISO 2022 IR 87"),
+    "\xd4\xcf\xc0\xde^\xc0\xdb\xb3\\\x1b$B;3ED\x1b(J^\x1b$BB@O:\x1b(J");
+  u = vtkDICOMValue(
+    vtkDICOMVR::PN, vtkDICOMCharacterSet("ISO_IR 13"),
+    "\xd4\xcf\xc0\xde^\xc0\xdb\xb3");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(
+    vtkDICOMVR::PN, vtkDICOMCharacterSet("ISO 2022 IR 13\\ISO 2022 IR 87"),
+    "\x1b$B;3ED\x1b(J^\x1b$BB@O:\x1b(J");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(
+    vtkDICOMVR::PN, vtkDICOMCharacterSet("ISO_IR 192"),
+    "\xef\xbe\x94\xef\xbe\x8f\xef\xbe\x80\xef\xbe\x9e^\xef\xbe\x80\xef\xbe\x9b\xef\xbd\xb3");
+  TestAssert(v.Matches(u));
+  u = vtkDICOMValue(
+    vtkDICOMVR::PN, vtkDICOMCharacterSet("ISO_IR 192"),
+    "\xe5\xb1\xb1\xe7\x94\xb0^\xe5\xa4\xaa\xe9\x83\x8e");
+  TestAssert(v.Matches(u));
   }
 
   return rval;
